package com.gic.haoban.manage.api.service.hm;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.haoban.manage.api.dto.hm.HmClerkRelationDTO;
import com.gic.haoban.manage.api.dto.hm.HmQrcodeDTO;
import com.gic.haoban.manage.api.dto.hm.HmQrcodeStoreDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmQrcodeQDTO;

import java.util.List;

/**
 * @author mozhu
 * @date 2022/7/5 09:33
 */
public interface HmQrcodeApiService {

    /**
     * 添加
     *
     * @param hmQrcodeQDTO hm qrcode qdto
     * @return int
     * @author mozhu
     * @date 2022-07-05 10:59:08
     */
    int add(HmQrcodeQDTO hmQrcodeQDTO);

    /**
     * 更新
     *
     * @param hmQrcodeQDTO hm qrcode qdto
     * @return int
     * @author mozhu
     * @date 2022-07-05 10:59:06
     */
    int update(HmQrcodeQDTO hmQrcodeQDTO);

    /**
     * 查询详情
     *
     * @param hmId hb id
     * @return {@link HmQrcodeDTO }
     * @author mozhu
     * @date 2022-07-05 10:59:03
     */
    HmQrcodeDTO queryById(Long hmId);


    /**
     * 作废
     *
     * @param hmId hm id
     * @return boolean
     * @author mozhu
     * @date 2022-07-06 13:40:27
     */
    boolean delById(Long hmId);

    /**
     * 导购解绑作废
     *
     * @param clerkId
     * @param enterpriseId 企业标识
     * @return boolean
     * @author mozhu
     * @date 2022-07-06 15:39:01
     */
    boolean delByClerkId(String clerkId,String enterpriseId);


    /**
     * 通过id添加num
     *
     * @param hmId    hm id
     * @param clerkId 职员id
     * @return int
     * @author mozhu
     * @date 2022-07-07 11:14:53
     */
    int updateAddNumById(Long hmId, String clerkId);


    /**
     * 重置 每天的上限好友数量
     * 定时任务
     *
     * @param param 参数
     * @author mozhu
     * @date 2022-07-07 13:52:28
     */
    void resetAddNumAndOverFlag(String param);


    /**
     * 检查员工开启联系我
     * 定时任务
     * @param param 参数
     * @author mozhu
     * @date 2022-07-07 14:25:54
     */
    void checkStaffOpenContact(String param);


    /**
     * 活码门店ids
     *
     * @param enterpriseId 企业标识
     * @return {@link List }<{@link HmQrcodeStoreDTO }>
     * @author mozhu
     * @date 2022-07-08 15:07:15
     */
    List<HmQrcodeStoreDTO> getHmStoreByEnterpriseId(String enterpriseId);

    /**
     * 根据导购信息获取hmid
     *
     * @param hmId hm id
     * @return {@link List }<{@link HmClerkRelationDTO }>
     * @author mozhu
     * @date 2022-07-13 17:09:56
     */
    List<HmClerkRelationDTO> getHmClerkByHmId(Long hmId);


    /**
     * 批量修改活码分组
     * @param hmIds
     * @param groupId
     * @return
     */
    ServiceResponse<Boolean> batchUpdateHmQrcodeGroup(List<Long> hmIds, Long groupId);
}
