package com.gic.haoban.manage.service.service.hm.impl;

import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.PageHelperUtils;
import com.gic.haoban.manage.api.dto.qdto.hm.WxUserAddLogQDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.WxUserAddLogSearchQDTO;
import com.gic.haoban.manage.service.dao.mapper.hm.WxUserAddLogMapper;
import com.gic.haoban.manage.service.entity.hm.TabHmLink;
import com.gic.haoban.manage.service.entity.hm.TabWxUserAddLog;
import com.gic.haoban.manage.service.pojo.bo.hm.HmLinkBO;
import com.gic.haoban.manage.service.pojo.bo.hm.HmQrcodeBO;
import com.gic.haoban.manage.service.pojo.bo.hm.WxUserAddLogBO;
import com.gic.haoban.manage.service.service.MemberQueryService;
import com.gic.haoban.manage.service.service.hm.HmLinkService;
import com.gic.haoban.manage.service.service.hm.HmQrcodeService;
import com.gic.haoban.manage.service.service.hm.WxUserAddLogService;
import com.gic.member.api.dto.es.MemberDataDTO;
import com.gic.member.tag.api.dto.MemberTagItemDTO;
import com.gic.member.tag.api.service.MemberTagApiService;
import com.github.pagehelper.PageHelper;

@Service("wxUserAddLogService")
public class WxUserAddLogServiceImpl implements WxUserAddLogService {

    private static final Logger log = LoggerFactory.getLogger(WxUserAddLogService.class);

    @Autowired
    private WxUserAddLogMapper wxUserAddLogMapper;
    @Autowired
    private HmLinkService hmLinkService;
    @Autowired
    private HmQrcodeService hmQrcodeService;
    @Autowired
    private MemberQueryService memberQueryService;
    @Autowired
    private MemberTagApiService memberTagApiService;

    public void save(WxUserAddLogQDTO qdto) {
        String externalUserid = qdto.getExternalUserid();
        String wxUserId = qdto.getWxUserId();
        TabWxUserAddLog entity = new TabWxUserAddLog();
        entity.setWxEnterpriseId(qdto.getWxEnterpriseId());
        entity.setEnterpriseId(qdto.getEnterpriseId());
        entity.setExternalUserid(externalUserid);
        entity.setAddChannel(0);
        entity.setStatusFlag(1);
        entity.setWxUserId(qdto.getWxUserId());
        entity.setStaffId(qdto.getStaffId());
        entity.setStaffName(qdto.getStaffName());
        if (qdto.isDelFlag()) {
            TabWxUserAddLog log = this.wxUserAddLogMapper.selectNewLog(externalUserid, wxUserId);
            entity = log;
            entity.setStatusFlag(2);
            // 是否所有好友删除 1是0否
            entity.setAllDelFlag(this.isAllDel(externalUserid));
        }
        // 新增好友
        if (!qdto.isDelFlag()) {
            String memberLabelId = null;
            if (null != qdto.getHmId()) {
                HmQrcodeBO hm = this.hmQrcodeService.queryById(qdto.getHmId());
                entity.setAddChannel(2);
                entity.setHmId(qdto.getHmId());
                entity.setHmName(hm.getName());
                entity.setClerkId(hm.getClerkId());
                entity.setClerkCode(hm.getClerkCode());
                entity.setClerkName(hm.getClerkName());
                entity.setStoreId(hm.getStoreId());
                entity.setStoreCode(hm.getStoreCode());
                entity.setStoreName(hm.getStoreName());
                memberLabelId = hm.getMemberLabelId();
            }
            if (null != qdto.getLinkId()) {
                HmLinkBO link = this.hmLinkService.getByLinkIdFormCache(qdto.getWxEnterpriseId(),
                        qdto.getEnterpriseId(), qdto.getLinkId());
                entity.setAddChannel(1);
                entity.setLinkId(qdto.getLinkId());
                entity.setLinkName(link.getName());
                memberLabelId = link.getMemberLabelId();
            }

            if (null != memberLabelId) {
                entity.setMemberLabelId(memberLabelId);
                MemberTagItemDTO dto = this.memberTagApiService.findMemberTagItem(memberLabelId);
                if (null != dto) {
                    entity.setMemberName(dto.getTagName() + dto.getTagItemName());
                }
            }
            if (null != qdto.getMemberId()) {
                entity = this.addMemberInfo(qdto.getEnterpriseId(), qdto.getMemberId(), entity);
            }
        }
        this.wxUserAddLogMapper.insert(entity);

    }

    // 冗余会员信息
    private TabWxUserAddLog addMemberInfo(String enterpriseId, String memberId, TabWxUserAddLog entity) {
        String returnFields = "memberId,photoUrl,name,nickName,cardNo,gradeId,gradeName,phoneNumber";
        JSONObject memberObj = this.memberQueryService.queryDataSingle(enterpriseId, memberId, returnFields);
        if (null == memberObj) {
            return entity;
        }
        MemberDataDTO memberDataDTO = memberObj.toJavaObject(MemberDataDTO.class);
        entity.setMemberId(memberId);
        entity.setMemberName(memberDataDTO.getName());
        entity.setMemberNick(memberDataDTO.getNickName());
        entity.setMemberImage(memberDataDTO.getPhotoUrl());
        entity.setMemberPhone(memberDataDTO.getPhoneNumber());
        entity.setMemberGradeId(memberDataDTO.getGradeId());
        entity.setMemberGradeName(memberDataDTO.getGradeName());
        entity.setMemberCardNo(memberDataDTO.getCardNo());
        return entity;
    }

    private int isAllDel(String externalUserid) {
        List<TabWxUserAddLog> list = this.wxUserAddLogMapper.listAllForDel(externalUserid);
        Map<String, List<TabWxUserAddLog>> map = list.stream()
                .collect(Collectors.groupingBy(TabWxUserAddLog::getWxUserId));
        Set<String> set = map.keySet();
        Iterator<String> it = set.iterator();
        boolean exist = false;
        while (it.hasNext()) {
            String wxUserId = it.next();
            List<TabWxUserAddLog> itemList = map.get(wxUserId);
            if (itemList.size() == 1 && itemList.get(0).getStatusFlag() == 2) {
                log.info("成员={}只有1条删除的记录", wxUserId);
            } else {
                TabWxUserAddLog addLog = itemList.get(0);
                if (addLog.getStatusFlag() == 1) {
                    log.info("成员={}最新是添加记录", wxUserId);
                    if (addLog.getWxUserId().equals(wxUserId)) {
                        log.info("成员={}本次删除", wxUserId);
                    } else {
                        exist = true;
                    }
                } else {
                    log.info("成员={}最新是删除记录", wxUserId);
                }
            }
        }
        if (exist) {
            return 0;
        }
        return 1;
    }

    @Override
    public Page<WxUserAddLogBO> page(WxUserAddLogSearchQDTO qdto, BasePageInfo pageInfo) {
        PageHelper.startPage(pageInfo);
        List<TabWxUserAddLog> list = this.wxUserAddLogMapper.listForPage(qdto);
        Page<WxUserAddLogBO> retPage = PageHelperUtils.changePageHelperToCurrentPage(list, WxUserAddLogBO.class);
        return retPage;
    }

}
